.TH NETRESTORE2.0 n  "15 Jul 95"  "Network Restoration Simulation System"  "NETRESTORE User Guide"

.SH NAME
\fBNETRESTORE2.0   \-   A NEtwork RESTORation Evaluation system\fP

.SH DESCRIPTION
.B NETRESTORE2.0
is a discrete event simulation system for evaluating DCS-based distributed 
network restoration algorithms. It includes the implementations of four 
prominent link restoration algorithm simulators: 
an enhanced version of Grover's Self Healing Network (SHN) algorithm, 
Bellcore's FITNESS, UCCS' RREACT and Two Prong.  These simulators were recently
modified from earlier versions to allow the simulation of large networks, 
additional simulation control, and extended simulation result reporting. 

Efficient data structures enable these simulators to handle 
a 99 node 228 span mci network.  The upper limit of the network size to be
simulated is in fact constrained only by the system's virtual memory and
the processor power.

The extended 
simulation control parameters include  the signalling channel transmission 
speed, message length, message processing delay, fiber refractive index, 
DCS cross connect time, DCS operating mode (parallel or sequential), 
span distance overwrite, and repeat limit.  
The user can change these parameters through the command line interface. 

Besides the basic restoration progress report on restoration paths, 
restoration level (bandwidth found divided by bandwidth lost),  and
restoration time, the analysis of the efficiency of the restoration methods
is extended to include Path Network Efficiency (PNE) and 
Path Length Efficiency (PLE).  PNE compares the bandwidth found by the
network restoration method being evaluated with the maximal bandwidth 
found by an optimal spare usage algorithm.  PLE compare the number of spares
used in the paths found by the restoration method being evaluated with that
generated by an optimal spare usage algorithm. \fBNETRESTORE2.0\fP implemented
a relaxation based optimal spare usage algorithm called \fBrx_mnet\fP and
the optimal spare usage results are read by the simulators to generate the 
PNE and PLE numbers.

The five networks are used to test these simulators. They include the
11 node 23 span latax network, 11 node 23 span nj network (with a much larger
working and spare channels than latax network), 28 node 47 span us network,
45 node 71 span layer network, and 99 node 228 span mci network.  
The simulation results of our enhanced SHN implementation on latax and us 
networks were verified against the results reported in Grover's dissertation.  
Except for the incorrect entries in Grover's dissertation, 
the results of our version was found to be consistent with those reported 
by Grover in terms of restoration level, restoration paths, 
and PNE efficiency numbers.
On all link restoration cases in the latax network, our version achieves 
100% PLE while there are five cases Grover's SHN does not reach 100% PLE.

The restoration times in our simulation results seems to be higher. In latax
network cases, it can be more than twice.  One of the reasons is that 
our simulation assumes a fixed 10 msec message processing time with 
possible additional queueing delay while Grover measured the performance of 
their logic on SUN3 machine with unknown context switching time in signature 
interrupt processing.  If the lower bound of Grover's measurement on 
signature processing time can be obtained, we can replace the 10 msec
message processing time via command line interface and compare the simulation
results again.  Details of the verification results are reported in 
SHN IMPLEMENTATION VERIFICATION SECTION.

To facilitate the use of \fBNETRESTORE2.0\fP, a set of file conversion
routines were developed to speed up the conversion of MCI file formats to
those used by \fBNETRESTORE2.0\fP.  The file conversion routines also
generate scripts for simulating every link break of the network on all four
simulators.  With these scripts, a user can type in a single command and
generate all the simulation results on a network with default simulation 
parameter setting.  For different parameter settings, a user can modify
these scripts using an editor.  

The rest of this user guide includes the following Sections:
Installation, Simulation Scenario, Network File Generation, Simulation Options,
Simulation Output Interpretation, and SHN Implementation Verification.
The Appendices include the man pages of all the software routines, 
samples of the script files, samples of the network files, and the 
summarized simulation results of the four simulators on the five test networks.
.SH INSTALLATION
This section provides the basic instructions for installing \fBNETRESTORE\fP
system.  It is assumed that the compressed tar file of \fBNETRESTORE\fP
software, \fInetres.tar.Z\fP, is available via ftp or cartridge. 
The \fInetres.tar.Z\fP is about 3 MBytes.
The installation requires about 20 MBytes disk space.

Following these steps to install \fBNETRESTORE\fP:

First create a directory for building the \fBNETRESTORE\fP system and 
assume that the path name to this directory is \fI$NETRES\fP.

Copy the compressed tar file, \fInetres.tar.Z\fP to \fI$NETRES\fP directory.
Change the working directory by issuing "cd $NETRES" command.

Uncompress \fInetres.tar.Z\fP by using the "uncompress netres.tar.Z" command.

Restore the directory structure by issuing the "tar xvf netres.tar" command.

Type "make install" to start the compilation of all the NETRESTORE programs.
The compiled programs are saved in \fI$NETRES/bin\fP directory.

The source code is organized in three sub-directories under \fI$NETRES/src\fP: 
\fIshn\fP, \fInetconv\fP, and \fIrelax\fP. The makefile is set up to compile 
source code in that order. The \fIshn\fP directory contains the source code of 
four simulators (
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and 
.BR "shnnew" ) ,
NETSIM discrete event simulation routines, and a legacy
p2c library. The \fInetconv\fP directory contains the source code of the file 
format conversion routines: 
.BR "nps2net" ,
.BR "np2net" ,
.BR "ns2net" ,
.BR "nps2dat" ,
.BR "np2dat" ,
and
.BR "ns2dat" .
The \fIrelax\fP directory contains \fIrelax.f\fP,
the relax fortran code, and the \fIrx_mnet.c\fP, a front end interface C code to
the relax program.  

After installation is complete, include the \fI$NETRES/bin\fP in the $PATH
environment variable and include \fI$NETRES/man\fP is the $MANPATH
environment variable of your .login or .cshrc file.  Perform "source .login"
or login again.  \fBNETRESTORE2.0\fP is now ready for use.

The makefile in each src directory assumes the use of cc. If your system
uses gcc, change the line "CC=cc" in the makefile to "CC=gcc".

The rx_mnet assumes system V UNIX, such as Solaris, and uses memset() to 
initialize some data structures. If you use Berkeley UNIX, you need to
replace memset() with bzero().  Future NETRESTORE release will use 
#ifndef to get around this problem.  

If the compilation of p2c package reports error on makeproto, it may be
caused by the old makeproto code from a different platform. Remove
the makeproto in \fI$NETRES/src/shn/p2c/src\fP and recompile again.

The current directory contains executable code generated on a SPARC20.
But unfortunately, our SPARC20 does not have f77 compiler. Therefore,
we were unable to test the compilation of f77 on relax.f.
The bin directory  does not contain the relax executable code. 
If you are using a SPARC20, after setting the bin directory in your search
paths, you only need to go to the \fIsrc/relax\fP directory and execute
the "make relax" command and then you are ready to test a network design.  
You do not have to execute the "make install" command.  
I would suggest
that a separate network directory is created to contain the results for a 
network design, similar to the arrangement under the net subdirectory here.
.SH NETRESTORE DIRECTORY ORGANIZATION
The \fI$NETRES\fP contains five directories, a makefile, and a README file.
The five directories include 
doc for user guide in WORD format, 
man for man pages, 
src for source code,
bin for executable programs, and
net for a set of test networks, related scripts and their simulation results, 
each in a sub-directory. The test networks includes latax, nj, us, layer,
mci, metro and whalen.

The README file in bin directory contains the brief description of 
the command syntax and a simple network testing scenario.
The net directory contains the network files, the related simulation scripts, 
and the simulation results of latax, nj, metro, whalen, us, layer, and 
mci networks.

Note the the four simulators have been tested on latax.net, nj.net, us.net,
and layer.net for all the link restoration cases. 
For mci.net, due to the long simulation time for the default repeat limit of 13,
we modify the scripts to run on repeat limit of 7. 
The new scripts have suffix 'nf' in their file names and 
do not generate files for the simulation results of individual cuts.
Two Prong runs pretty fast.
fitness second, rreact and shnnew are very slow. shnnew is the slowest (take
the whole day).
Potentially there are still bugs undetected.
Please report bugs to chow@quandary.uccs.edu.
.SH SIMULATION SCENARIO
In order to analyze a network using 
.BR "NETRESTORE" ,
a four step sequence will generally be followed.  Each of the
steps below is explained in more detail in later sections by the
same name.
.sp
1)  Generate Network Description Files
.sp
First use one of nps2net, np2net, or ns2net to generate .net file for the 
network and the scripts for simulating the restoration of all its link breaks. 
Modify the .net if necessary. Possibilities include the integration of the
span distance information, node specific repeat limit, or node specific 
message processing time.
For example, the following command is used to generate mci.net file:
.sp
\fBnps2net mcinodes.txt mcipaths.txt mcispares.txt mci.net 0.01\fP
.sp
2) Compute the optimal spare usage.
.sp
First use one of nps2dat, np2dat, or ns2dat to generate .dat file by executing
.sp
\fBnps2dat mcinodes.txt mcipaths.txt mcispares.txt mci.dat\fP
.sp
Then run 
.sp
\fBrx_mnet -s mcinodes.txt mci.dat 2\fP
.sp
to generate the restoration paths with optimal spare usage
for each link break.
.sp
3) Run Script For Simulation Results.
.sp
Simply type
.sp
\fBmci.scr\fP
.sp
The simulation results will be in mci.fit.res, mci.rr.res, mci.twop.res, and
mci.shn.res files.  Here fit stands for fitness, rr for rreact, twop for 
twoprong, and shn for shnnew.
.sp
4)  Interpret Simulation Output.
.sp
See detailed description in SIMULATION OUTPUT INTERPRETATION section.
.sp
.sp
5)  Select Other Simulation Options
.sp
For example, try to see the effect of reducing the node processing time from 
the default 0.01 seconds to 0.001 seconds.  The command line simulation option 
for that is \fB/p=0.001\fP.
.sp
6)  Edit Scripts to Incorporate the Changes in Simulation Options.
.sp
Type "vi mci.fit.scr". Use ":1,$s/> /\\/p=0.001 > /g" to update all the
fitness commands in mci.fit.scr. Note that since `/' has special meaning
in the vi substitute command, we need to use `\\' quoting character to 
avoid that interpretation.
.sp
7) Return to Step 3).
.sp
.SH NETWORK FILE GENERATION
A set of companion tools is available to facilitate
the conversion of existing network files into a NETRESTORE file format shared 
by the four NETRESTORE simulators. 
They provide the basic statistics of the generated network file and 
generate the script file for the simulation of each link restoration in the 
network.  A required input parameter for node processing delay is shared by
all these file conversion utilities.
.BR "nps2net" 
takes the node, path, and spare files as input parameters. 
.BR "np2net"
takes the node and path files as input parameters, and allows users to specify
the spare to working ratio for every span as an additional input parameter.
The node file contains a list of node specification lines with four fields: 
sequence number, node ID, V and H coordinates.
The path file contains the current traffic pattern with a node list of each
of working path followed by `45', DS3 bandwidth, in each line.
These two programs correlate the working  and spare channels in each span, and
the internal ports of each node from the path file.
.BR "ns2net"
takes the node and span files as input parameters. 
The span file contains
the span specification lines, which specify the working and span channels in
each span, and the span distance in kilo meters. 
The 
.BR "ns2net"
correlates the number of input ports in each node from the span 
files.
Check the `man page' for  
.BR "nps2net" ,   
.BR "np2net" ,
and 
.BR "ns2net" .
The
.B nps2net
file is located in   \fI$NETRES/bin\fP.  The command for creating mci.net is: 
.sp
\fBnps2net  mcinodes.txt  mcipaths.txt  spares.txt  mci.net  0.01\fP
.sp
Three files are required as input (nodes, paths, and spares) 
followed by the name of the target .net file and
the default message processing delay (in seconds).

To change the message processing time of a specific node,
the node specific repeat limit, or the span distance
of a span, users are referred to the `man page' for the 
.BR "NETRESTORE file format" .
The current version of simulators is case sensitive on the .net filename.
Please use all lower cases for the .net filenames.
.SH SIMULATION OPTIONS
Options and parameters available with \fBshnnew\fP are outlined below.  Each 
option is invoked using the first character (lower case) of the option 
in question.

.B 1TSTIME
\-   This option relates to the transmission time of each individual
signature.  For those
wishing a more fine grained analysis, the node signature transmission
time can be set to something other than the default 0.0 seconds.  For
example, a delay of 2 msec would be set using:     /1=0.002

.B 2INDEX
\-   Refractive index of the optical fiber can be specified with this
option, which defaults to 1.5, corresponding to a 5 microsecond
delay per Km.  It may be changed to 1.4 using:     /2=1.4

.B BREAK
\-   Break is the means by which link failures are specified.  
The format is /b=<node1>\-<node2>.  
This is a \fIrequired\fP parameter.  The user specifies the end nodes using 
the same names as are contained in the companion `.net' network file.        
Note that the name of a node, N07, is different from N7 and node 00 is 
interpreted different from node 0. 
For the break between N02 and N00, use:
	 /b=N02-N01.

.B CROSS_CONNECT_TIME
\-   This is the time delay for a node to effect DCS cross connect.  A default time of  0.01 seconds is assumed.  An 8 msec time would be specified as:     /c=0.008

.B DEBUG
\-   The DEBUG option is not interactive, but simply provides more
detail on the restoration process.  Specifically, it includes the
output of additional simulation information to a log file (default
NETSIM.OUT), a complete
copy of the path listing as found in the .net file, as well as a list of
specific path IDs which were disrupted.  Details of the broadcast phase
are also included.  To invoke this option, add the  `/d' qualifier to the
simulation command.

.B INTERACTIVE
\-   More extensive diagnosis information on fault recovery progress is 
possible using the interactive mode.  Details of events within each node are
reported, for example.  The user is given the option to continue
examining events, abort, or skip `n' events before stopping for another
visual check.  Interactive operation is invoked using:     /i

.B MSG_LENGTH
\-   Restoration time depends on the average length (bytes) 
of restoration messages exchanged by distributed nodes.  The
default message length is 8 bytes.  Restoration message length could be changed to 12 bytes using:     /m=12

.B OUT
\-   Simulation results are stored in a default log file called 
NETSIM.OUT.  An alternative file name can be specified using:	/out=<file_name>

.B PPTOVERRIDE
\-   The default node processing time for each signature
(distinct from 1STSTIME) used in
".net" files created with NETRESTORE translation utilities is ten msec.
This overrides the node processing time of individual nodes, read in as
the third parameter of the first line of each node description
in the .net file.  To override the node processing time 
to 8 msec, use:     /p=0.008

.B REPEATS
\-   This parameter represents the maximum repeat limit or hop count limit
(13 is default) for restoration requests in the broadcast phase.  
It is observed that increasing repeat limit does not guarantee the increase in
restoration level with a real-time constrain. The increase message volume 
may cause network congestion and results in slower restoration time and lower
restoration level.
Specify 9 hop limit as:     /r=9

.B STOPTIME
\-   Upper limit on simulation runtime (sec).  Default value 
is ten (10.0) seconds.  A five second runtime limit is 
specified as:     /s=5.0

.B TIMEOUT
\-   In the event that the Sender and Chooser are unable to achieve full
restoration, the Sender will time-out and cancel the restoration
request, releasing reserved bandwidth.  A default timeout value
of ten (10.0) seconds is assumed.  To get a timeout of 5 seconds, use:     /t=5.0

.B WRITE_LINK
\-   The default is to use the link distance field 
from the .net file.  Specifying  /w  as an option will cause the 
simulator to calculate the link distance 
(and hence link delay) according to V&H or pseudo latitude 
and longitude coordinates as appropriate. 

.B X_SPEED
\-   Transmission speed (kbps) for restoration messages on all links can
be set with this option.  A default value of  8 kbps  is assumed.  A transmission speed of 384 kbps is specified as:     /x=384

.B Y_PARALLEL_DCS_MODE
\-   The default is batch processing mode where the DCS connections of all the 
found bandwidth in a message processing can be done in one CROSS_CONNECT_TIME.
/y=1 specifies the sequential DCS processing mode where each 
channel connection is done in one CROSS_CONNECT_TIME. /y=k, where
k is an integer greater than 1, specifies the parallel DCS processing mode where
the found bandwidth are connected in k channels as a batch and each k channels
are connected in CROSS_CONNECT_TIME.
A parallel DCS processing of connecting 4 channels in a batch is specified as:     /y=4
.br
Note that all the simulators implement this option except shnnew which finds
and connects individual spare paths one at a time.
.BP
.SH SIMULATION OUTPUT INTERPRETATION
.SH SHN Simulation Output
The first portion of the simulation output documents the principal
options selected for this specific run.  Next,
.B shnnew
reports status (4 phases) as it checks the integrity of the .net file,
and ends by indicating the number of active paths before link failure.
Simulation output starts following the statement "SHN algorithm
results...".  

The end nodes of the user specified link failure are
listed, followed by the bandwidth needed for full restoration.

Restoration progress is reported by way of two record types.  An example
of the first record type:

msg_count=00100@ 0.05232 sec Buf#=00102 bwRes=0 
                                         SenderRes=0000 ChooserRes=0002
.TP 0
The first field reports a count of restoration messages which the 
simulator has
processed and the associated elapsed restoration time in
seconds.  Buf# reports the cumulative number of pending restoration 
messages buffered by the nodes.  bwRes reports the cumulative bandwidth restored at this point.  Finally, the amount of reserved bandwidth from the perspective
of the Sender and Chooser is given.

The second record type used to report simulation progress highlights a
path restoration event.  An example follows:
.TP 3
signal 114 rstrd at 252.403 msec (  5.56%)--->
(N00(P0053XP00042)-N02(P00037XP00014)-N01(P00037XP00055)).
.TP 0
In this instance, signal 114 is reported as being restored after an 
elapsed time of 252.403 msec.  The accumulated percentage of restoration
level is also
indicated.  Signal 114 corresponds to a specific working path 
listed in the last half
of the .net file.  Here, a restoration path is found that originates 
at node N00, 
passes through node N02, and terminates at node N01.  In fact,
this is what is reported in the last part of the record.  The arrow
simply indicates that the balance of the record appears on the following
line.  DCS cross connect details for signal 114 are given in the last half 
of the record.  For example, in node N00, ports 53 and 42 are connected.

The last several statements in the simulation output provide a summary
of the simulation.  For example, in the simulation run on us.net with N01-N06 
cut, the following statements appears:
.TP 0
Last Non Time Out Event Time = 0.562081
.nf
Total msg_count=991
access file us.dat.ls/N01_N06cut.l
optimal bandwidth that can be restored=7
optimal spare used =39
N01-N06 6/100=  6.00% @0.541085 msgs=985/991 PNE=6/7 PLE=39/33

.fi
"Last Non Time Out Event Time" reports the total
elapsed restoration time, including cleanup traffic.  The total number
of restoration messages corresponding to this time is given by "Total
msg_count".  
The next three lines indicates that the \fBshnnew\fP found the file with
optimal spare usage and read in the optimal restorable bandwidth and
the minimum spare used to achieve the optimal restoration level.

The last line of output again summarizes the end nodes
connected to the failed link, restored bandwidth, the lost bandwidth, 
(also expressed as a percent).  The time at which this level of restoration 
was achieved is reported.  The two numbers after "msgs=" represent 
respectively the number of messages needed to restore bandwidth, and the 
total number of messages required by the algorithm (including cleanup).
PNE stands for Path Number Efficiency. It is a measure for evaluating the
restoration level achieved by a network restoration method.  It contains
two numbers. The first number represents the number of paths restored by
the network restoration method being evaluated. The second number represents
the maximum number of paths that can be restored. It was typically computed by 
an optimal algorithm called relax and read in by \fBshnnew\fP.   
PLE stands for Path Length Efficiency. It is a measure for evaluating the 
efficiency of a network restoration method in terms of the spare usage. It
contains two numbers. The first number is the total number of spares in 
the paths found by the optimal spare usage algorithm.  The second number
is the total number of spares in the paths found by the network restoration
method being evaluated.  Note that the PLE ratio is only meaningful when
the PNE=1. The higher the PLE ratio the better. 

Here the 
\fBshnnew\fP
only restore 6 channels while the optimal algorithm found 7 channels.
In the N20-N22 cut of 
us.net, the last line of simulation output shows:

N20-N22 12/100= 12.00% @0.990109 msgs=1403/1403 PNE=12/12 PLE=40/44

Here the optimal algorithm found paths that use only 40 spares, 
while \fBshnnew\fP found paths that use 44 spares.

\fBshnnew\fP also appends this summary information line to the file 
\fInetres.shn.res\fP in the working directory.  This allows a script with
all the span breaks to accumulate the summarized network restoration results.

.SH Fitness, Two Prong, Rreact Simulation Output Interpretation
Since the simulation outputs of these three simulators are almost identical,
we will use a simulation output of Two Prong for the illustration.

Simulation output starts with the statement "TWOPRONG algorithm
results...with <net_file_name>".  
The first portion of the simulation output documents the principal
options selected for this specific run.  Next,
.B twoprong
reads in the .net file and  checks its integrity. For example, the total 
number of working ports of a node matches that implies by the working channels 
in the node's spans.
The end nodes of the user specified link failure are
listed, followed by the bandwidth needed for full restoration.

Restoration progress is reported by way of one record types.  An example
of the progress report with the simulation of the following command,
.sp
\fBtwoprong nj.net /b=N07-n04 /y=1\fP
.sp
is as follows:
.sp
.nf
Path  1 (16 bw) 19.75% rstrd at 290.80 msec ---> (N07 N06 N04)
Path  2 (27 bw) 53.09% rstrd at 560.80 msec ---> (N07 N05 N04)
Path  3 (17 bw) 74.07% rstrd at 730.80 msec ---> (N07 N10 N04)
Path  4 ( 1 bw) 75.31% rstrd at 740.80 msec ---> (N07 N05 N00 N02 N04)
Path  5 ( 3 bw) 79.01% rstrd at 770.80 msec ---> (N07 N03 N00 N02 N04)
Path  6 (11 bw) 92.59% rstrd at 880.80 msec ---> (N07 N03 N00 N02 N04)
Path  7 ( 6 bw)100.00% rstrd at 940.80 msec ---> (N07 N03 N00 N01 N02 N04)
access file nj.dat.ls/N04_N07cut.l
optimal bandwidth that can be restored=81
optimal spare used =204
N07-N04 81/81=100.00% @0.940800 msgs=142/183 PNE=81/81 PLE=204/210
.fi
.sp
In this instance, the DCS' are operated at the sequential
processing mode, as it is indicated  by the option /y=1.
Each channel is connected with the default CROSS_CONNECT_TIME=0.01 seconds by 
the DCS'. 
Path 1 is reported as being restored after an 
elapsed time of 290.80 msec.  It restored 16 channels and the accumulated 
percentage of restoration level is indicated as 19.75%.  
Specifically, a restoration path is found that originates from node N07, 
passes through node N06, and terminates at node N04.  In fact,
this is what is reported in the last part of the record pointed by an arrow.
Note that here the Sender, N04, received the the ack message of 16 channels as
early as 130.80 msec. But since its DCS is operated at the sequential 
processing mode with each channel connected at 0.01 seconds interval, 
Path 1 does not restore the last of 16 channels until 130.80+16*10=290.80 msec.

The last line of output again summarizes the end nodes
connected to the failed link, restored bandwidth, the lost bandwidth, 
(also expressed as a percent).  The time at which this level of restoration 
was achieved is reported.  The two numbers after "msgs=" represent 
respectively the number of messages needed to restore bandwidth, and the 
total number of messages required by the algorithm (including cleanup).
PNE stands for Path Number Efficiency. It is a measure for evaluating the
restoration level achieved by a network restoration method.  It contains
two numbers. The first number represents the number of paths restored by
the network restoration method being evaluated. The second number represents
the maximum number of paths that can be restored. It was typically computed by 
an optimal algorithm called \fBrx_mnet\fP and read in by \fBtwoprong\fP.   
PLE stands for Path Length Efficiency. It is a measure for evaluating the 
efficiency of a network restoration method in terms of the spare usage. It
contains two numbers. The first number is the total number of spares in 
the paths found by the optimal spare usage algorithm.  The second number
is the total number of spares in the paths found by the network restoration
method being evaluated.  Note that the PLE ratio is only meaningful when
the PNE=1. The higher the PLE ratio the better. 

Here \fBtwoprong\fP
restore 81 channels the same as that found by the optimal algorithm.
But \fBtwoprong\fP found paths that use 210 while 
\fBrx_mnet\fP found paths that use 204 spares.

\fBtwoprong\fP also appends this summary information line to the file 
\fInetres.fit.res\fP in the working directory.  This allows a script with
all the span breaks to accumulate the summarized network restoration results.
.SH SHN IMPLEMENTATION VERIFICATION
The simulation results of our enhanced SHN implementation on latax and us 
networks were verified against the results reported in Grover's dissertation.  
Table 1 shows the comparison results on latax network.
Table 2 shows the comparison results on us network.

.nf
     Table 1: PERFORMANCE COMPARISON ON LATAX NETWORK 

        BW    RESTORATION      RESTORATION
SPAN    LOST  ---LEVEL------   -----TIME------  ----PNE-----  ----PLE-----
              UCCS    GROVER   UCCS     GROVER  UCCS  GROVER  UCCS  GROVER
==========================================================================
N00-N02 18    38.89%  38.89%   0.234350 0.144    7/7   7/7    14/14  14/14
N00-N03 18    61.11%  61.11%   0.693326 0.462   11/11 11/11   31/31  31/33*-
N00-N04 13   100.00% 100.00%   0.613551 0.254   13/13 13/13   29/29  29/29
N00-N05 14    78.57%  78.57%   0.562875 0.306   11/11 11/11   28/28  28/28
N00-N07 13   100.00% 100.00%   0.672825 0.360   13/13 13/13   26/26  26/27*
N01-N00 18    27.78%  27.78%   0.342851 0.167    5/5   5/5    10/10  10/10
N01-N02 13    38.46%  38.46%   0.282650 0.218    5/5   5/5    10/10  10/10
N02-N04  4   100.00% 100.00%   0.512976 0.177    4/4   4/4     8/8    8/8
N03-N07 15   100.00% 100.00%   0.622875 0.352   15/15 15/15   34/34  34/38*
N03-N08 13   100.00% 100.00%   0.720251 0.361   13/13 13/13   34/34  34/34
N04-N03 17   100.00% 100.00%   1.024525 0.294   17/17 17/17   36/36  36/43*
N04-N06  4    75.00%  75.00%   0.304500 0.146    3/3   3/3     6/6    6/6
N04-N07 20   100.00% 100.00%   0.904500 0.259   20/20 20/20   40/40  40/40
N04-N08 12   100.00% 100.00%   0.704525 0.163   12/12 12/12   24/24  24/24
N04-N10 12   100.00% 100.00%   0.884600 0.259   12/12 12/12   24/24  24/24
N05-N04 12    66.67%  66.67%   0.633075 0.191    8/8   8/8    16/16  16/16
N05-N07 12    91.67%  91.67%   0.723326 0.365   11/11 11/11   27/27  36/37*!
N06-N07 10    20.00%  20.00%   0.412900 0.189    2/2   2/2     4/4    4/6*
N07-N08 14   100.00% 100.00%   0.623175 0.250   14/14 14/14   28/28  28/28
N07-N10 16   100.00% 100.00%   0.704600 0.465   16/16 16/16   37/37  37/41*
N08-N09 20    10.00%  10.00%   0.144340 0.141    2/2   2/2     4/4    4/4
N08-N10 16   100.00% 100.00%   0.543076 0.288   16/16 16/16   33/33  33/33
N10-N09 9     77.78%  77.78%   0.314340 0.240    7/7   7/7    14/14  14/14

*: Five entries where UCCS' SHN have better PLE numbers than Grover's.
-: Table 9.2 of Grover's dissertation has 32/33 instead of 31/33.
!: Table 9.2 of Grover's dissertation has 36/37 instead of 27/28.



     Table 2: PERFORMANCE COMPARISON ON US NETWORK 

        BW     RESTORATION      RESTORATION
SPAN    LOST  ---LEVEL------   -----TIME------  ----PNE-----  ----PLE-----
              UCCS    GROVER   UCCS     GROVER  UCCS  GROVER  UCCS  GROVER
==========================================================================
N01-N00 100    1.00%   1.00%   0.235044 0.2068   1/1   1/1     4/4    4/4
N02-N01 100    5.00%   6.00%   0.663105 0.5163   5/6   6/6    37/26  37/37-
N03-N02 100    5.00%   6.00%   0.424061 0.7341   5/6   6/6    37/26  37/37
N04-N00 100    1.00%   1.00%   0.235044 0.2069   1/1   1/1     4/4    4/4
N05-N04 100    5.00%   5.00%   0.277024 0.2071   5/5   5/5    12/12  12/12
N06-N01 100    6.00%   7.00%   0.544088 0.3700   6/7   7/7    39/33  39/39
N06-N05 100    5.00%   6.00%   0.395051 0.3200   5/6   6/6    33/25  33/33-
N07-N03 100    9.00%  10.00%   0.378034 0.5407   9/10 10/10   43/34  43/43-
N07-N06 100    6.00%   7.00%   0.377065 0.2919   7/7   7/7    36/36  36/36
N08-N04 100    5.00%   5.00%   0.366054 0.2665   5/5   5/5    13/13  13/13
N08-N05 100    5.00%   5.00%   0.198027 0.1818   5/5   5/5    11/11  11/11
N09-N06 100    5.00%   6.00%   0.459071 0.3304   5/6   6/6    26/21  26/26-
N09-N08 100    1.00%   2.00%   0.194034 0.3259   1/2   2/2    11/3   11/11
N10-N09 100    1.00%   3.00%   0.249037 0.4207   1/3   3/3    18/4   18/18-
N11-N07 100    6.00%  10.00%   0.416054 0.8781   6/10 10/10   55/23  55/69-
N11-N10 100    5.00%   6.00%   0.621078 0.6328   5/7   6/7    42/27  42/34-
N12-N03 100    2.00%   4.00%   0.231031 0.3676   2/4   4/4    20/6   20/20-
N13-N07 100    6.00%   8.00%   0.442048 0.3207   6/10  8/10   51/21  51/33-
N13-N12 100    6.00%   6.00%   0.515092 0.3474   6/6   6/6    36/36  36/36
N14-N11 100    8.00%   8.00%   0.909119 0.6827   8/8   8/8    41/41  41/41
N14-N13 100    7.00%   5.00%   0.729119 0.1923   7/8   5/8    37/29  37/13*
N15-N13 100    8.00%   8.00%   0.989119 0.4637   8/8   8/8    40/40  40/40
N16-N15 100    6.00%   6.00%   0.341024 0.2432   6/6   6/6    18/18  18/18
N17-N16 100    6.00%   6.00%   0.338027 0.2095   6/6   6/6    18/18  18/18
N18-N15 100   10.00%   6.00%   0.499010 0.2086  10/10  6/10   25/25  25/13*
N18-N17 100   10.00%  10.00%   0.332020 0.2036  10/10 10/10   25/25  25/25
N19-N15 100   14.00%  14.00%   0.459010 0.2159  14/14 14/14   37/37  37/45
N19-N17 100   11.00%   7.00%   0.482020 0.2459  11/11  7/11   28/28  28/20*
N19-N18 100   10.00%   6.00%   0.427017 0.1596  10/10  6/10   20/20  20/12*
N20-N13 100    7.00%   7.00%   0.628075 0.4069   7/7   7/7    32/32  32/32
N20-N14 100    7.00%   7.00%   0.555085 0.4039   7/7   7/7    29/29  29/31#
N20-N15 100    7.00%   7.00%   0.447024 0.2506   7/7   7/7    24/24  24/24
N21-N10 100    8.00%   8.00%   0.519078 0.3628   8/8   8/8    33/33  33/33
N21-N20 100   10.00%  11.00%   0.489064 0.4092  10/11 11/11   48/42  48/48-
N22-N20 100   12.00%  11.00%   0.980109 0.6141  12/12 11/12   40/44  40/42*
N22-N21 100   10.00%  10.00%   0.569010 0.2759  10/10 10/10   25/25  25/25
N23-N15 100   12.00%  12.00%   0.541051 0.3093  12/12 12/12   33/33  33/33
N23-N19 100    9.00%  10.00%   0.508034 0.3943   9/10 10/10   39/33  39/39-
N23-N21 100   11.00%  11.00%   0.533030 0.2537  11/11 11/11   35/36  35/36
N23-N22 100    8.00%   8.00%   0.521064 0.3569   8/8   8/8    26/26  26/26
N24-N10 100    5.00%   5.00%   0.603098 0.3952   5/5   5/5    29/29  29/29
N25-N21 100    7.00%   7.00%   0.461058 0.2876   7/7   7/7    29/29  29/29
N25-N24 100    2.00%   3.00%   0.363037 0.3162   2/3   3/3    15/9   15/15-
N26-N22 100    4.00%   4.00%   0.317044 0.2333   4/4   4/4    14/14  14/14
N26-N25 100    5.00%   6.00%   0.415044 0.2309   5/6   6/6    19/15  19/19-
N27-N25 100    5.00%   3.00%   0.342041 0.1534   5/5   3/5    14/14  14/6 *
N27-N26 100    1.00%   -----   0.133024 ------   1/1   ---     2/2   -----?

*: There are five cases UCCS' SHN found more paths than Grover's SHN.
#: Same number of paths were found but UCCS' SHN use less spares.
-: There are 17 cases Grover's SHN found more paths than UCCS' SHN.
?: entry missing in Table 9.3 of Grover's dissertation.
.fi

The simulation results in Tables 9.2 and 9.3 Grover's dissertation only shows
the time the path is found and not include the DCS cross connect time.
We have changed our scripts to use /c=0.0 option for zero cross connection
time.  This allows the simulator to report the path finding time. 
The us.net in Figure 9.2 of Grover's dissertation does not show the
numbers of spares between nodes 15 and 18, between nodes 15 and 19,
and between nodes 18 and 19, and between nodes 3 and 7.  By observing
the results in Table 9.3 of Grover's dissertation 
we conclude that the number of spares between nodes 
15 and 18 is 5, the number of spares between nodes 15 and 19 is 8,
the number of spares between nodes 18 and 19 is 5, 
the number of spares between nodes 3 and 7 is 2.

For latax network,
except for the incorrect entries in Grover's dissertation, 
the results of our version was found to be consistent with those reported 
by Grover in terms of restoration level, restoration paths, 
and PNE efficiency numbers.
On all link restoration cases in the latax network, our version achieves 
100% PLE while there are five cases Grover's SHN does not reach 100% PLE.

For us network,  in 17 cases Grover's SHN found more paths than UCCS' SHN;
in five cases, UCCS' SHN found more paths than Grover's SHN. There is one
case where both found the same number of paths but UCCS' SHN used less spares.
Note that in this network node specific repeat limit is used. SHN will use
the repeat limit of the sender (shown in Figure 9.2 on top of node label)
in its signature processing.  The optimal spare usage results used in PNE
and PLE numbers are generated with no repeat limits. 

The restoration times in our simulation results seems to be higher. In latax
network cases, it can be more than twice.  One of the reasons is that 
our simulation assumes a fixed 10 msec message processing time with 
possible additional queueing delay while Grover measured the performance of 
their logic on SUN3 machine with unknown context switching time in signature 
interrupt processing.  If the lower bound of Grover's measurement on 
signature processing time can be obtained, we can replace the 10 msec
message processing time via command line interface and compare the simulation
results again.  

Over all, UCCS' SHN simulation performs very close to Grover's SHN.
With the improvement on signature cancellation phase, UCCS' SHN will
have fast clean up time since it prevents the request of 
the old index to chase the bandwidth just being released with the same index.
The 17 cases in us network where Grover's SHN performs better than UCCS' SHN
is currently being investigated.

.SH BUG REPORT
Please send bug reports, including the platform on which 
.B shnnew 
is executed and the specific scenario which produced the bug,
to chow@quandary.uccs.edu.  We will try to reply as early as possible.
