.TH np2dat n  "20 Jul 95"  "np2dat file converter"  "NETRESTORE Reference Manual"

.SH NAME
\fBnp2dat   \-   convert Nodes and Paths files to "relax" .dat file format\fP

.SH SYNTAX
\fBnp2dat  <node_file>.txt  <path_file>.txt  <net_file>.dat spare-to-working_ratio\fP

.SH DESCRIPTION
.B np2dat
is a tool which converts the MCI 
.B node 
file and the associated 
.B path 
file (path entries end in 45) 
into the 
.B relax.dat
file format for computing optimal spare usage.      
A constant spare-to-working link ratio (0.25 if spare channel number = 
0.25*ceil(working channel number))
is the last parameter in the command string.  ceil() is the ceiling function.
.B np2dat
also prints the basic network statistics.

As an example, \fImci1.dat\fP is generated by the following command:
.sp
\fBnp2dat mcinodes.txt mcipaths.txt mci1.dat 0.25\fP
.sp
and it is then used in the command 
.sp
\fBrx_mnet -s mcinodes.txt mci1.dat 2\fP
.sp
to generate the optimal spare usage information, including
files of optimal spare paths for each span cut in \fImci1.dat.ls\fP directory
and the file, \fImci1.out\fP, with the summary information.

The NETRESTORE family of tools includes:
\
network restoration simulators such as
\
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and
.BR "shnnew" ;
a relax-based optimal spare algorithm called
.BR "rx_mnet" ;
\
and network file conversion utilities to facilitate the creation of
network description files, such as
\
.BR "nps2net" ,
.BR "np2net" ,
.BR "ns2net" ,
.BR "np2dat" ,
.BR "np2dat" ,
and
.BR "ns2dat" .

The
.B np2dat
program resides in the  \fI$NETRES/bin\fP
directory and its source code resides in the \fI$NETRES/src/netconv\fP.  

Since the simulators will generate a lot of files for the simulation results
and use temporary working files in the current working directory,
it is suggested that a separate directory be created for the simulation study 
of a network.  
This allows files of different networks to be better organized
and allows multiple users to work concurrently on the same computer (in 
different working directories.)
The suitably formatted input files should be placed
in this directory before running the conversion utility.  Specific
formats for each of the input files are described in the following
sections.

.SH COMMON FILE FORMAT CONVENTIONS
All network parameters on same line of input file should be considered 
in a free field format, i.e., use whitespace as a field delimiter.

.SH NODE FILE FORMAT
The first line of the node file contains the name of the file.
The rest of line entries in the node file contain four fields, which are
respectively:  `sequenceNo',  `nodeID',  `V', and  `H'.  V and H represent
the vertical and horizontal coordinates of the node.  The sequence
number starts with 1 for the first entry, and is incremented for each
additional entry.  
The sequenceNo is also used as the internal ID of the node.
The nodeID is the string name of the node to be specified.
The current upper limit is 300 nodes and 1500 spans.
One node specification entry per line.

.nf
Field Description    			Data Type                  Usual Value  
---------------------------------------------------------------------------
Node sequence number			POSITIVE INTEGER            
Network node ID      			STRING, size 15            1-3 digits
Vertical coordinate  			REAL                       or latitude
Horizontal coordinate			REAL                       or longitude

.fi
.SH PATH FILE FORMAT
The path file describes the working traffic in the network.
No blank lines are permitted.
The first line of the path file contains the file name.
The rest of the path file is the path specification with one path per line.
Each path is defined as a sequence of valid nodeIDs (from nodeFile)
separated by white space.  
The node sequence should start with the originating path
node, and end with the last node in the path.  Intermediate nodes must
be in the order in which they would be visited traveling the path.  An
important constraint is that all path strings must terminate in `45'.
This represents the DS3 data rate in Mbps, and is used as a delimiter.
Path strings are currently limited to 255 characters in length.

.nf
Field Description                   	Data Type              Usual Value
---------------------------------------------------------------------------
Originating Node ID          		STRING, size 15            1-3 digits
Next node ID in node sequence		STRING, size 15            1-3 digits
           :					     :
Last node ID in node sequence		STRING, size 15            1-3 digits
Sentinel value  (in Mbps)    		POSITIVE INTEGER           45

.fi
.SH RELAX .dat FILE FORMAT
The first line of the relax data file contains a number indicating whether
this network is bidirectional or not. 0 for non-bidirectional, others for 
directional.
The second line of the relax data file contains the number of the nodes in
the network. The third line of the relax data file contains the number of
uni-directional link specification lines.
Each link specification line has six fields: link sequence #, originating node
Internal ID, terminating node internal ID, link cost, # of spares, # of working channels.

.nf
Field Description   			Data Type        
---------------------------------------------------------------------------
Link Sequence #				POSITIVE INTEGER
Originating Node Internal ID		POSITIVE INTEGER           1-3 digits
Terminating Node Internal ID		POSITIVE INTEGER           1-3 digits
Link Cost				INTEGER
Number of Spares			POSITIVE INTEGER
Number of Working Channels		POSITIVE INTEGER
.fi
.sp
Note that here the internal node ID is used. It corresponds to the 
sequence number of the node in the node file or the internal node ID in .net
file. \fBrelax\fP, a fortran code,  will not accept 0 as internal node 
ID due to the array indexing.  This is also the reason why the node file is 
needed as one of the input parameter in \fBrx_mnet\fP to convert the results 
expressed in terms of internal node IDs back to the equivalent string names.
The order of spare and working channels is also different from that
in .net file.  For a bi-directional link network, the file lists two 
uni-directional links of the same span as a pair.
For example the first link specification line of mci1.dat is 
.in 1i
.sp
.nf
1 88 97 1 28 111
.sp
.in -1i
.fi
Here 28 is equal to ceil(111). 111 is the accumulated working channels
in the span between nodes 88 and 97
.
.SH BUG REPORT
Please send bug reports, including the platform on which
.B np2dat
is executed and the specific scenario which produced the bug,
to chow@quandary.uccs.edu.  We will try to reply as early as possible.
