.TH np2net n  "20 Jul 95"  "np2net file converter"  "NETRESTORE Reference Manual"

.SH NAME
\fBnp2net   \-   Convert Nodes and Paths to NETRESTORE .net file format\fP

.SH SYNTAX
.TP 0
\fBnp2net  <node_file>.txt  <path_file>.txt  <net_file>.net   
.in 2i
msg_proc_delay spare-to-working_ratio\fP

.SH DESCRIPTION
.B np2net
is a utility program which takes two files in MCI format as input 
representing lists of the network 
.B "nodes"  
and
.BR "paths"  
respectively, and outputs a 
.BR ".net" 
file in NETRESTORE format.  All nodes in the 
network are assumed to have the same message processing delay, which 
is entered as a single parameter (units of seconds) at the end of 
the command string.  Users can specify the spare-to-working ratio
(0.25 if spare channel number = 0.25* ceil(working channel number)).
\ 
.B np2net
also generates script files that runs  
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and
.B "shnnew"
(with <net_file>.fit.scr, <net_file>.rr.scr, <net_file>.twop.scr, 
and <net_file>.shn.scr as file names) on every span cut.
These scripts save the simulation results of individual span restorations
to directory with the simulator's name.  Each script also generates a 
summary information file with <net_file>.<abb>.res as a file name. Here
<abb> is an abbreviated name of a simulator: fit for fitness, rr for rreact,
twop for twoprong, and shn for shnnew.  The old .res files are copied to
.res.bak files as back up.
A script file, <net_file>.scr, for running all these four scripts are also
created.  

As an example, \fImci1.net\fP is created by the following command:
.sp
\fBnp2net mcinodes.txt mcipaths.txt mci.net 0.01 0.25\fP
.sp
Here 0.01 is the message processing delay and 0.25 is the spare to working
channels ratio.  The default file type is V_H2.
The same command also creates \fImci1.top\fP, 
which has the same content as the mci1.net but the destination
internal node ID in the link specification is replaced by the string name of
the node to make it easier to read the network topology.  
It also generates \fImci1.scr\fP, \fImci1.fit.scr\fP, 
\fImci1.rr.scr\fP, \fImci1.twop.scr\fP,
and \fImci1.shn.scr\fP scripts files to facilitate the simulation.
The \fImci1.scr\fP is a script file for executing the above four scripts.

The NETRESTORE family of tools includes:
\
network restoration simulators such as
\
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and
.BR "shnnew" ;
a relax-based optimal spare algorithm called
.BR "relax" ;
\
and network file conversion utilities to facilitate the creation of
network description files, such as
\
.BR "np2net" ,
.BR "np2net" ,
.BR "ns2net" ,
.BR "nps2dat" ,
.BR "np2dat" ,
and
.BR "ns2dat" .

The
.B np2net
program reside in the \fI$NETRES/bin\fP
directory and its source code resides in the \fI$NETRES/src/netconv\fP.  
Since the simulators will generate a lot of files for the simulation results
and use temporary working files in the current working directory,
it is suggested that a separate directory be created for the simulation study
of a network.
This allows files of different networks to be better organized
and allows multiple users concurrently working on the same system (in
different working directories.)
The suitably formatted input files should be placed
in this directory before running the conversion utility.  Specific
formats for each of the input files are described in the following
sections.
.SH COMMON FILE FORMAT CONVENTIONS
All network parameters on same line of input file should be considered 
in a free field format, i.e., use whitespace as a field delimiter.

.SH NODE FILE FORMAT
The first line of the node file contains the name of the files.
The rest of line entries in the node file contain four fields, which are
respectively:  `sequenceNo',  `nodeID',  `V', and  `H'.  V and H represent
the vertical and horizontal coordinates of the node.  The sequence
number starts with 1 for the first entry, and is incremented for each
additional entry.  The nodeID is a string name of the node to be specified.
The current upper limit is 300 nodes and 1500 spans.
One node specification entry per line.

.nf
Field Description    			Data Type                  Usual Value  
---------------------------------------------------------------------------
Node sequence number			POSITIVE INTEGER            
Network node ID      			STRING, size 15            1-3 digits
Vertical coordinate  			REAL                       or latitude
Horizontal coordinate			REAL                       or longitude

.fi
.SH PATH FILE FORMAT
The path file describes the working traffic in the network.
No blank lines are permitted.
The first line of the path file contains the file name.
The rest of the path file is the path specification with one path per line.
Each path is defined as a sequence of valid nodeIDs (from nodeFile)
separated by white space.  
The node sequence should start with the originating path
node, and end with the last node in the path.  Intermediate nodes must
be in the order in which they would be visited traveling the path.  An
important constraint is that all path strings must terminate in `45'.
This represents the DS3 data rate in Mbps, and is used as a delimiter.
Path strings are currently limited to 255 characters in length.

.nf
Field Description                   	Data Type              Usual Value
---------------------------------------------------------------------------
Originating Node ID          		STRING, size 15            1-3 digits
Next node ID in path sequence		STRING, size 15            1-3 digits
           :					     :
Last node ID in path sequence		STRING, size 15            1-3 digits
Sentinel value  (in Mbps)    		POSITIVE INTEGER           45

.fi
.SH NETRESTORE .net FILE FORMAT
The NETRESTORE .net file format captures the the network topology and status
information that is needed to run the simulators provided by NETRESTORE 
system. The first line contains the number of network nodes and the file type.
The file types include L_L2, V_H2, L_L2wSpanKms, and V_H2wSpanMiles. 
`2' in these file types indicates that these are new NETRESTORE version 2 file 
format.  In this new version of file format, the internal, working, and spare
port numbers are included in the node specification so that the simulators
can allocate more data structures more efficiently and allow large networks
to be simulated. `L_L' in these file types indicates that the longitude and 
latitude coordinate system is used to specify the location of a node.
`V_H' in these file types indicates that the conventional telecommunications
V&H coordinate system is used to specify the location of a node.
`wSpanKms' and `wSpanMiles' indicate that the link specification of the 
file contains link distance field in Kilo meters or Miles units.
The simulators can be told to overwrite these link distance fields and use
the locations of the end nodes in a span to calculate the link distance.

The second line starts the node-span specification followed by the traffic
specification. The node-span specification describes the information of a node
and its connected spans.  It consists of a group of
the specification line of a node followed by a list of the span specification 
line of its spans. The traffic specification describe the current traffic
pattern. It consists of the number of paths followed by a list of the path 
specification lines.   The following is the detail field specification.
.sp
.nf
Line#  Description			Data Type           Usual Value
------------------------------------------------------------------------------

0      Number of network nodes, N	POSITIVE INTEGER
       Type of coordinates		STRING,              L_L2 or V_H2
						   	  or V_H2wSpanMiles

1+0    Node identification string	STRING, size 15       3 char
       Number of internodal links, Li	POSITIVE INTEGER
       Node msg processing delay	POSITIVE REAL         0.010 sec
       Node specific repeat limit	POSITIVE INTEGER      0 (default)
       Node location, pseudo-latitude	REAL              or V coordinate
       Node location, pseudo-longitude	REAL              or H coordinate
       Number of Working Ports		POSITIVE INTEGER    
       Number of Spare   Ports		POSITIVE INTEGER    
       Number of Internal Ports		POSITIVE INTEGER    
       Number of Total Ports		POSITIVE INTEGER    
       Internal node ID			POSITIVE INTEGER      

1+1        Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]
	   *** Link Distance field only exist for V_H2wSpanMiles type file.
           .
           .
1+Li       Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]
       .
       .
       .

N+0    Node identification string	STRING, size 15       3 char
       Number of internodal links, Lx	POSITIVE INTEGER
       Node msg processing delay	POSITIVE REAL         0.01 sec
       Node specific repeat limit	POSITIVE INTEGER     0(default)
       Node location, pseudo-latitude	REAL              or V coordinate
       Node location, pseudo-longitude	REAL              or H coordinate
       Number of Working Ports		POSITIVE INTEGER    
       Number of Spare   Ports		POSITIVE INTEGER    
       Number of Internal Ports		POSITIVE INTEGER    
       Number of Total Ports		POSITIVE INTEGER    
       Internal node ID			POSITIVE INTEGER      

N+1        Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]
	   *** Link Distance field only exist for V_H2wSpanMiles type file.
           .
           .
N+Lx       Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]


T     Number of Paths				POSITIVE INTEGER

T+1   Path Sequence Number			POSITIVE INTEGER
      Originating node name			STRING, size 15       3 char
      Next node name in path sequence		STRING, size 15       3 char
                :                                   :
      Last node name in path sequence		STRING, size 15       3 char
           .
           .
           .
T+p   Path Sequence Number			POSITIVE INTEGER
      Originating node name			STRING, size 15       3 char
      Next node name in path sequence		STRING, size 15       3 char
                :                                   :
      Last node name in path sequence		STRING, size 15       3 char
.sp
.fi
.SH BUG REPORT
Please send bug reports, including the platform on which
.B np2net
is executed and the specific scenario which produced the bug,
to chow@quandary.uccs.edu.  We will try to reply as early as possible.
