.TH ns2dat n  "20 Jul 95"  "ns2dat file converter"  "NETRESTORE Reference Manual"

.SH NAME
\fBns2dat   \-   convert Nodes and Spans files to "relax" .dat file format\fP

.SH SYNTAX
\fBns2dat   <node_file>.txt   <span_file>.txt   <net_file>.dat\fP

.SH DESCRIPTION
.B ns2dat
is a utility program which converts the MCI 
.B node 
file (using V&H coordinates) and the associated 
.B span 
file (span entries with end nodes, span sequence number, the numbers of 
working and spare channels) into the 
.B relax.dat
file format for computing optimal spare usage.      
\ 
.B ns2dat
also prints the basic network statistics.

As an example, \fIlayer.dat\fP is generated by the following command:
.sp
\fBns2dat   layernodes.txt   layerspans.txt   layer.dat\fP
.sp
and it is then used in the command 
.sp
\fBrx_mnet   -s   layernodes.txt   layer.dat   2\fP
.sp
to generate the optimal spare usage information, including
files of optimal spare paths for each span cut in \fIlayer.dat.ls\fP directory
and the file, \fIlayer.out\fP, with the summary information.

The NETRESTORE family of tools includes:
\
network restoration simulators such as
\
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and
.BR "shnnew" ;
a relax-based optimal spare algorithm called
.BR "rx_mnet" ;
\
and network file conversion utilities to facilitate the creation of
network description files, such as
\
.BR "nps2net" ,
.BR "np2net" ,
.BR "ns2net" ,
.BR "ns2dat" ,
.BR "np2dat" ,
and
.BR "ns2dat" .

The
.B ns2dat
program resides in the  \fI$NETRES/bin\fP
directory and its source code resides in the \fI$NETRES/src/netconv\fP.  

Since the simulators will generate a lot of files for the simulation results
and use temporary working files in the current working directory,
it is suggested that a separate directory be created for the simulation study 
of a network.  This allows files of different networks to be better organized
and allows multiple users to work concurrently on the same computer (in 
different working directories.)
The suitably formatted input files should be placed
in this directory before running the conversion utility.  Specific
formats for each of the input files are described in the following
sections.

.SH COMMON FILE FORMAT CONVENTIONS
All network parameters on same line of input file should be considered 
in a free field format, i.e., use whitespace as a field delimiter.

.SH NODE FILE FORMAT
The first line of the node file contains the name of the file.
The rest of line entries in the node file contain four fields, which are
respectively:  `sequenceNo',  `nodeID',  `V', and  `H'.  V and H represent
the vertical and horizontal coordinates of the node.  The sequence
number starts with 1 for the first entry, and is incremented for each
additional entry.  The nodeID is the string name of the node to be specified.
The current upper limit is 300 nodes and 1500 spans.
One node specification entry per line.

.nf
Field Description    			Data Type                  Usual Value  
---------------------------------------------------------------------------
Node sequence number			POSITIVE INTEGER            
Network node ID      			STRING, size 15            1-3 digits
Vertical coordinate  			REAL                       or latitude
Horizontal coordinate			REAL                       or longitude

.fi
.SH SPAN FILE FORMAT
The span file describes the span distance and the working and spare capacity 
of the network.
The first five lines of the span file is the header describing each
field in the span specification line.
No blank lines are permitted.
The rest of the span file is the span specification with one span per line.
Each span specification has six fields: the originating node ID, 
the terminating node ID, the span sequence number, the number of working 
channels, the number of spare channels, and the span distance in Miles.

.nf
Field Description                   	Data Type              Usual Value
---------------------------------------------------------------------------
Originating Node ID          		STRING, size 15            1-3 digits
Terminating Node ID          		STRING, size 15            1-3 digits
Span Sequence Number         		POSITIVE INTEGER           
Number of Working Channels   		POSITIVE INTEGER           
Number of Spare Channels     		POSITIVE INTEGER           
Span Distance in Miles              	REAL
.fi
.SH RELAX .dat FILE FORMAT
The first line of the relax data file contains a number indicating whether
this network is bidirectional or not. 0 for non-bidirectional, others for 
directional.
The second line of the relax data file contains the number of the nodes in
the network. The third line of the relax data file contains the number of
uni-directional link specification lines.
Each link specification line has six fields: link sequence #, originating node
ID, terminating node ID, link cost, # of spares, # of working channels.

.nf
Field Description   			Data Type        
---------------------------------------------------------------------------
Link Sequence #				POSITIVE INTEGER
Originating Node Internal ID		POSITIVE INTEGER           1-3 digits
Terminating Node Internal ID		POSITIVE INTEGER           1-3 digits
Link Cost				INTEGER
Number of Spares			POSITIVE INTEGER
Number of Working Channels		POSITIVE INTEGER
.fi
.sp
Note that here the internal node ID is used. It corresponds to the
sequence number of the node in the node file or the internal node ID in .net
file. \fBrelax\fP, a fortran code,  will not accept 0 as internal node
ID due to the array indexing.  This is also the reason why the node file is 
needed as one of the input parameter in \fBrx_mnet\fP to convert the results
expressed in terms of internal node IDs back to the equivalent string names.
The order of spare and working channels is also different from that
in .net file.  For a bi-directional link network, the file lists two
uni-directional links of the same span as a pair.
.SH BUG REPORT
Please send bug reports, including the platform on which
.B ns2dat
is executed and the specific scenario which produced the bug,
to chow@quandary.uccs.edu.  We will try to reply as early as possible.
