.TH ns2net n  "20 Jul 95"  "ns2net file converter"  "NETRESTORE Reference Manual"

.SH NAME
\fBns2net   \-   Convert Nodes and Spans to NETRESTORE .net file format\fP

.SH SYNTAX
.TP 0
\fBns2net   <node_file>.txt  <spans_file>.txt <net_file>.net  msg_proc_delay\fP

.SH DESCRIPTION
.B ns2net
is a program which converts the
.B node
file (using V&H coordinates) and
the associated
.B span
file (with working, spare, and span miles)
into NETRESTORE
.B network
file format.
All nodes in the 
network are assumed to have the same message processing delay, which 
is entered as a single parameter (units of seconds) at the end of 
the command string.    
\ 
.B ns2net
also generates script files that runs  
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and
.B "shnnew"
(with <net_file>.fit.scr, <net_file>.rr.scr, <net_file>.twop.scr, 
and <net_file>.shn.scr as file names) on every span cut.
These scripts save the simulation results of individual span restorations
to directory with the simulator's name.  Each script also generates a 
summary information file with <net_file>.<abb>.res as a file name. Here
<abb> is an abbreviated name of a simulator: fit for \fBfitness\fP, rr for 
\fBrreact\fP, twop for \fBtwoprong\fP, and shn for \fBshnnew\fP.  
The old .res files are copied to
.res.bak files as back up.
A script file, <net_file>.scr, for running all these four scripts are also
created.  

As an example, \fIlayer.net\fP is created by the following command:
.sp
\fBns2net   layernodes.txt   layerspans.txt   layer.net   0.01\fP
.sp
Here 0.01 is the message processing delay.  The default file type is V_H2.
The same command also creates \fIlayer.top\fP, 
which has the same content as the \fIlayer.net\fP but the destination
internal node ID in the link specification is replaced by the string name of
the node to make it easier to read the network topology.
It also generates \fIlayer.scr\fP, \fIlayer.fit.scr\fP, 
\fIlayer.rr.scr\fP, \fIlayer.twop.scr\fP,
and \fIlayer.shn.scr\fP scripts files to facilitate the simulation.
The \fIlayer.scr\fP is a script file for executing the above four scripts.

The NETRESTORE family of tools includes:
\
network restoration simulators such as
\
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and
.BR "shnnew" ;
a relax-based optimal spare algorithm called
.BR "relax" ;
\
and network file conversion utilities to facilitate the creation of
network description files, such as
\
.BR "ns2net" ,
.BR "np2net" ,
.BR "ns2net" ,
.BR "nps2dat" ,
.BR "np2dat" ,
and
.BR "ns2dat" .

The
.B ns2net
program reside in the \fI$NETRES/bin\fP
directory and its source code resides in the \fI$NETRES/src/netconv\fP.  
Since the simulators will generate a lot of files for the simulation results
and use temporary working files in the current working directory,
it is suggested that a separate directory be created for the simulation study
of a network.
This allows files of different networks to be better organized
and allows multiple users concurrently working on the same system (in
different working directories.)
The suitably formatted input files should be placed
in this directory before running the conversion utility.  Specific
formats for each of the input files are described in the following
sections.
.SH COMMON FILE FORMAT CONVENTIONS
All network parameters on same line of input file should be considered 
in a free field format, i.e., use whitespace as a field delimiter.

.SH NODE FILE FORMAT
The first line of the node file contains the name of the files.
The rest of line entries in the node file contain four fields, which are
respectively:  `sequenceNo',  `nodeID',  `V', and  `H'.  V and H represent
the vertical and horizontal coordinates of the node.  The sequence
number starts with 1 for the first entry, and is incremented for each
additional entry.  The nodeID is a string name of the node to be specified.
The current upper limit is 300 nodes and 1500 spans.
One node specification entry per line.

.nf
Field Description    			Data Type                  Usual Value  
---------------------------------------------------------------------------
Node sequence number			POSITIVE INTEGER            
Network node ID      			STRING, size 15            1-3 digits
Vertical coordinate  			REAL                       or latitude
Horizontal coordinate			REAL                       or longitude

.fi
.SH SPAN FILE FORMAT
The span file describes the span distance and the working and spare capacity 
of the network.
The first five lines of the span file is the header describing each
field in the span specification line.
No blank lines are permitted.
The rest of the span file is the span specification with one span per line.
Each span specification has six fields: the originating node ID, 
the terminating node ID, the span sequence number, the number of working 
channels, the number of spare channels, and the span distance in Miles.

.nf
Field Description                   	Data Type              Usual Value
---------------------------------------------------------------------------
Originating Node ID          		STRING, size 15            1-3 digits
Terminating Node ID          		STRING, size 15            1-3 digits
Span Sequence Number         		POSITIVE INTEGER           
Number of Working Channels   		POSITIVE INTEGER           
Number of Spare Channels     		POSITIVE INTEGER           
Span Distance in Miles              	REAL
.fi
.SH NETRESTORE .net FILE FORMAT
The NETRESTORE .net file format captures the the network topology and status
information that is needed to run the simulators provided by NETRESTORE 
system. The first line contains the number of network nodes and the file type.
The file types include L_L2, V_H2, L_L2wSpanKms, and V_H2wSpanMiles. 
`2' in these file types indicates that these are new NETRESTORE version 2 file 
format.  In this new version of file format, the internal, working, and spare
port numbers are included in the node specification so that the simulators
can allocate more data structures more efficiently and allow large networks
to be simulated. `L_L' in these file types indicates that the longitude and 
latitude coordinate system is used to specify the location of a node.
`V_H' in these file types indicates that the conventional telecommunications
V&H coordinate system is used to specify the location of a node.
`wSpanKms' and `wSpanMiles' indicate that the link specification of the 
file contains link distance field in Kilo meters or Miles units.
The simulators can be told to overwrite these link distance fields and use
the locations of the end nodes in a span to calculate the link distance.

The second line starts the node-span specification followed by the traffic
specification. The node-span specification describes the information of a node
and its connected spans.  It consists of a group of
the specification line of a node followed by a list of the span specification 
line of its spans. The traffic specification describe the current traffic
pattern. It consists of the number of paths followed by a list of the path 
specification lines.   The following is the detail field specification.
.sp
.nf
Line#  Description			Data Type           Usual Value
------------------------------------------------------------------------------

0      Number of network nodes, N	POSITIVE INTEGER
       Type of coordinates		STRING,              L_L2 or V_H2
						   	  or V_H2wSpanMiles

1+0    Node identification string	STRING, size 15       3 char
       Number of internodal links, Li	POSITIVE INTEGER
       Node msg processing delay	POSITIVE REAL         0.010 sec
       Node specific repeat limit	POSITIVE INTEGER      0 (default)
       Node location, pseudo-latitude	REAL              or V coordinate
       Node location, pseudo-longitude	REAL              or H coordinate
       Number of Working Ports		POSITIVE INTEGER    
       Number of Spare   Ports		POSITIVE INTEGER    
       Number of Internal Ports		POSITIVE INTEGER    
       Number of Total Ports		POSITIVE INTEGER    
       Internal node ID			POSITIVE INTEGER      

1+1        Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]
	   *** Link Distance field only exist for V_H2wSpanMiles type file.
           .
           .
1+Li       Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]
       .
       .
       .

N+0    Node identification string	STRING, size 15       3 char
       Number of internodal links, Lx	POSITIVE INTEGER
       Node msg processing delay	POSITIVE REAL         0.01 sec
       Node specific repeat limit	POSITIVE INTEGER     0(default)
       Node location, pseudo-latitude	REAL              or V coordinate
       Node location, pseudo-longitude	REAL              or H coordinate
       Number of Working Ports		POSITIVE INTEGER    
       Number of Spare   Ports		POSITIVE INTEGER    
       Number of Internal Ports		POSITIVE INTEGER    
       Number of Total Ports		POSITIVE INTEGER    
       Internal node ID			POSITIVE INTEGER      

N+1        Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]
	   *** Link Distance field only exist for V_H2wSpanMiles type file.
           .
           .
N+Lx       Link destination node, internal ID	POSITIVE INTEGER
           Link active flag			0 or 1                1
           Number of working channels		POSITIVE INTEGER
           Number of spare channels		POSITIVE INTEGER
           [Link Distance			POSITIVE INTEGER      in km]


T     Number of Paths				POSITIVE INTEGER

T+1   Path Sequence Number			POSITIVE INTEGER
      Originating node name			STRING, size 15       3 char
      Next node name in path sequence		STRING, size 15       3 char
                :                                   :
      Last node name in path sequence		STRING, size 15       3 char
           .
           .
           .
T+p   Path Sequence Number			POSITIVE INTEGER
      Originating node name			STRING, size 15       3 char
      Next node name in path sequence		STRING, size 15       3 char
                :                                   :
      Last node name in path sequence		STRING, size 15       3 char
.sp
.fi
.SH BUG REPORT
Please send bug reports, including the platform on which
.B ns2net
is executed and the specific scenario which produced the bug,
to chow@quandary.uccs.edu.  We will try to reply as early as possible.
