.TH rreact n  "19 Jul 95"  "RREACT Simulator"  "NETRESTORE  Reference Manual"
.SH NAME
\fBrreact\-simulator of a distributed protocol for Rapid REstoration of Active Communications Trunks\fP
.SH SYNTAX
\fBrreact  <net_file_name>.net  /b=nodeX\-nodeY  [ /options ]\fP

Simulation Options Section (below) explains available options.  As an example,
to simulate link restoration between nodes N03 and N08, 
with repeat limit of 14 and network restoration signaling channels 
operated at 192 kbps,
in a network described in file `my_network.net', the command is:
.sp
.in 1i
\fBrreact   my_network.net   /b=N03\-N08  /r=14  /x=192\fP
.in -1i
.SH DESCRIPTION
.B rreact  
is a distributed network restoration simulator which implements an 
version of Chow and McCaughey's algorithm called 
a distributed protocol for Rapid REstoration of Active Communications Trunks.
\fBrreact\fP follows the Sender-Chooser
paradigm and the Sender sends an aggregated bandwidth request along each 
outgoing active span that contains spares.  The tandem node floods the received
request along each outgoing active span that contains spares. 
The request messages contain the information of the traversed path, 
including the node list and the spares in each span along the path. 
They are therefore of variable length. 
With the additional path information, the Chooser is able to build "smart" 
picture of the current local network topology and status. This "smart" 
current network information can be used in optimization options or enhancements
and allows \fBrreact\fP to find shorter paths and use the spares
wisely. \fBrreact\fP establishes all restoration paths in one wave of request 
messages.  The main disadvantage of \fBrreact\fP is that its message volume is 
proportional to the number of possible paths between the Sender and the 
Chooser, which grows exponentially with the network size. Heuristics are
implemented to control the message volume during the flooding phase. 
This includes the use of hopcounts and discarding requests when their
traverse time passed half of the real-time requirement.

The simulator operates on a network description file.  
The user specifies a network link 
break as well as desired options, and the simulator provides a step-by-step
profile of the recovery process including the restored bandwidth, 
percentage of the restoration level, and associated time at which that level of 
restoration was achieved.

The NETRESTORE family of tools includes: network restoration simulators, 
such as,
.BR "rreact" ,
.BR "rreact" ,
.BR "twoprong" ,
and
.BR "shn" ,
a relaxation method based optimal spare algorithm called 
.BR "rx_mnet" ,
and network file conversion utilities to facilitate the creation of network
description files, such as
.BR "nps2net" ,
.BR "np2net" ,
.BR "ns2net" ,
.BR "nps2dat" ,
.BR "np2dat" , 
and
.BR "ns2dat" .
.SH SIMULATION SCENARIO
In order to analyze a network using the 
.B rreact
simulator, a four step sequence will generally be followed.  Each of the
steps below is explained in more detail in later sections by the
same name.
.TP 15
1)  Generate Network Description File
.TP 15
2)  Select Simulation Options
.TP 15
3)  Assemble Simulation Command with Options, Run Simulation
.TP 15
4)  Interpret Simulation Output.
.SH NETWORK FILE GENERATION
A set of companion tools is available to facilitate
the conversion of existing network files into a NETRESTORE file format shared 
by 
.BR "rreact" 
and other NETRESTORE simulators. 
They provide the basic statistics of the generated network file and 
generate the script file for the simulation of each link restoration in the 
network.  A required input parameter for node processing delay is shared by
all these file conversion utilities.
.BR "nps2net" 
takes the node, path, and spare files as input parameters. 
.BR "np2net"
takes the node and path files as input parameters, and allows users to specify
the spare to working ratio for every span as an additional input parameter.
The node file contains a list of node specification lines with four fields: 
sequence number, node ID, V and H coordinates.
The path file contains the current traffic pattern with a node list of each
of working path followed by `45', DS3 bandwidth, in each line.
These two programs correlate the working  and spare channels in each span, and
the internal ports of each node from the path file.
.BR "ns2net"
takes the node and span files as input parameters. 
The span file contains
the span specification lines, which specify the working and span channels in
each span, and the span distance in kilo meters. 
The 
.BR "ns2net"
correlates the number of input ports in each node from the span 
files.
Check the `man page' for  
.BR "nps2net" ,   
.BR "np2net" ,
and 
.BR "ns2net" .
The
.B nps2net
file is located in   \fI$NETRES/bin\fP.  

As an example, the command for creating mci.net is: 
.sp
\fBnps2net  mcinodes.txt  mcipaths.txt  mcispares.txt  mci.net  0.01\fP
.sp
Three files are required as input (nodes, paths, and spares) 
followed by the name of the target .net file and
the default message processing delay (in seconds).

To change the message processing time of a specific node,
the node specific repeat limit, or the span distance
of a span, users are referred to the `man page' for the 
.BR "NETRESTORE file format" .
The current version of simulators is case sensitive on the .net filename.
Please use all lower cases for the .net filenames.
.SH SIMULATION OPTIONS
Options and parameters available with \fBrreact\fP are outlined below.  Each 
option is invoked using the first character (lower case) of the option 
in question.

.B 2INDEX
\-   Refractive index of the optical fiber can be specified with this
option, which defaults to 1.5, corresponding to a 5 microsecond
delay per Km.  It may be changed to 1.4 using:     /2=1.4

.B BREAK
\-   Break is the means by which link failures are specified.  
This is a \fIrequired\fP parameter.  The user specifies the end nodes using 
the same names as are contained in the companion `.net' network file.        /b=<node1>\-<node2>

.B CROSS_CONNECT_TIME
\-   This is the time delay for a node to effect DCS cross connect.  A default time of  0.01 seconds is assumed.  An 8 msec time would be specified as:     /c=0.008

.B DEBUG
\-   The DEBUG option is not interactive, but simply provides more
detail on the restoration process.  Specifically, it includes 
the output of additional simulation information to a log file, by default
NETSIM.OUT, and a complete
copy of the path listing as found in the .net file, as well as a list of
specific path IDs which were disrupted.  Details of the broadcast phase
are also included.  To invoke this option, add the  `/d' qualifier to the
simulation command.

.B MSG_LENGTH
\-   Restoration time depends on the average length (bytes) 
of restoration messages exchanged by distributed nodes.  The
default message length is 8 bytes.  Restoration message length could be changed to 12 bytes using:     /m=12

.B OUT
\-   Simulation results are stored in a default log file called 
NETSIM.OUT.  An alternative file name can be specified using:	/out=<file_name>

.B PPTOVERRIDE
\-   The default protocol processing time or node processing times for each 
message are specified in the node specification lines of individual nodes
in the NETRESTORE file. 
This is the third parameter of the first line of each node description
in the .net file.  To override the message processing times of all the nodes 
to 8 msec, use:     p/=0.008

.B REPEATS
\-   This parameter represents the maximum repeat limit or hop count limit
(13 is default) for restoration requests in the broadcast phase.  
It is observed that increasing repeat limit does not guarantee the increase in
restoration level with a real-time constrain. The increase message volume 
may cause network congestion and results in slower restoration time and lower
restoration level.
Specify 9 hop limit as:     /r=9

.B STOPTIME
\-   Upper limit on simulation runtime (sec).  Default value 
is ten (10.0) seconds.  A short five second runtime limit is specified as:     /s=5.0

.B TIMEOUT
\-   In the event that the Sender and Chooser are unable to achieve full
restoration, the Sender will time-out and cancel the restoration
request, releasing reserved bandwidth.  A default timeout value
of ten (10.0) seconds is assumed.  To get a timeout of 5 seconds, use:     /t=5.0

.B WRITE_LINK
\-   The default is to use the link distance field 
in the .net file to calculate the link propagation delay. 
Specifying /w as an option will cause the simulator to calculate the link 
distance (and hence link delay) according to V&H or pseudo latitude and 
longitude coordinates as appropriate.  

.B X_SPEED
\-   Transmission speed (kbps) for restoration messages on all links can
be set with this option.  A default value of  8 kbps  is assumed.  A transmission speed of 384 kbps is specified as:     /x=384

.B Y_PARALLEL_DCS_MODE
\-   The default is batch processing mode where the DCS connections of all the 
found bandwidth in a message processing can be done in one CROSS_CONNECT_TIME.
/y=1 specifies the sequential DCS processing mode where each 
channel connection is done in one CROSS_CONNECT_TIME. /y=k, where
k is an integer greater than 1, specifies the parallel DCS processing mode where
the found bandwidth are connected in k channels as a batch and each k channels
are connected in CROSS_CONNECT_TIME.
A parallel DCS processing of connecting 4 channels in a batch is specified as:     /y=4
.br
Note that all the simulators implement this option except shnnew which finds
and connects individual spare paths one at a time.
.SH SIMULATION OUTPUT INTERPRETATION
Simulation output starts with the statement "RREACT algorithm
results...with <net_file_name>".  
The first portion of the simulation output documents the principal
options selected for this specific run.  Next,
.B rreact
reads in the .net file and  checks its integrity. For example, the total 
number of working ports of a node matches that implies by the working channels 
in the node's spans.
The end nodes of the user specified link failure are
listed, followed by the bandwidth needed for full restoration.

Restoration progress is reported by way of one record types.  An example
of the progress report with the simulation of the following command,
.sp
\fBrreact   nj.net   /b=N07-N04   /y=1\fP
.sp
is as follows:
.sp
.nf
Path  1 (16 bw) 19.75% rstrd@296.60msec -->(N07 N06 N04 )
Path  2 (27 bw) 53.09% rstrd@566.60msec -->(N07 N05 N04 )
Path  3 (17 bw) 74.07% rstrd@736.60msec -->(N07 N10 N04 )
Path  4 (17 bw) 95.06% rstrd@906.60msec -->(N07 N03 N08 N10 N04 )
Path  5 ( 1 bw) 96.30% rstrd@916.60msec -->(N07 N05 N00 N02 N04 )
Path  6 ( 3 bw) 100.00% rstrd@946.60msec -->(N07 N03 N00 N02 N04 )
access file nj.dat.ls/N04_N07cut.l
optimal bandwidth that can be restored=81
optimal spare used =204
Unrestored channels = 0
N07-N04 81/81=100.00% @0.946600 msgs=74/75 PNE=81/81 PLE=204/204
.fi
.sp
In this instance, the DCS' are operated at the sequential
processing mode, as it is indicated  by the option /y=1.
Each channel is connected with the default CROSS_CONNECT_TIME=0.01 seconds by 
the DCS'. 
Path 1 is reported as being restored after an 
elapsed time of 296.60 msec.  It restored 16 channels and the accumulated 
percentage of restoration level is indicated as 19.75%.  
Specifically, a restoration path is found that originates from node N07, 
passes through node N06, and terminates at node N04.  In fact,
this is what is reported in the last part of the record pointed by an arrow.
Note that here the Sender, N04, received the the ack message of 16 channels as
early as 136.60 msec. But since its DCS is operated at the sequential 
processing mode with each channel connected at 0.01 seconds interval, 
Path 1 does not restore the last of 16 channels until 136.60+16*10=296.60 msec.

The last line of output again summarizes the end nodes
connected to the failed link, restored bandwidth, the lost bandwidth, 
(also expressed as a percent).  The time at which this level of restoration 
was achieved is reported.  The two numbers after "msgs=" represent 
respectively the number of messages needed to restore bandwidth, and the 
total number of messages required by the algorithm (including cleanup).
PNE stands for Path Number Efficiency. It is a measure for evaluating the
restoration level achieved by a network restoration method.  It contains
two numbers. The first number represents the number of paths restored by
the network restoration method being evaluated. The second number represents
the maximum number of paths that can be restored. It was typically computed by 
an optimal algorithm called \fBrx_mnet\fP and read in by \fBrreact\fP.   
PLE stands for Path Length Efficiency. It is a measure for evaluating the 
efficiency of a network restoration method in terms of the spare usage. It
contains two numbers. The first number is the total number of spares in 
the paths found by the optimal spare usage algorithm.  The second number
is the total number of spares in the paths found by the network restoration
method being evaluated.  Note that the PLE ratio is only meaningful when
the PNE=1. The higher the PLE ratio the better. 

Here \fBrreact\fP
restore 81 channels the same as that found by the optimal algorithm.
Also \fBrreact\fP found paths that use 204 spares the same as that 
found by \fBrx_mnet\fP.

\fBrreact\fP also appends this summary information line to the file 
\fInetres.rr.res\fP in the working directory.  This allows a script with
all the span breaks to accumulate the summarized network restoration results.
.SH BUG REPORT
Please send bug report with the platform where \fBrreact\fP is executed and 
the specific scenario where the bug occurred to chow@quandary.uccs.edu. 
We will try to reply as early as possible.
