.TH rx_mnet n  "21 Jul 95"  "RELAX-BASED Optimal Spare Algorithm"  "NETRESTORE  Reference Manual"

.SH NAME
\fBrx_mnet   \-   relax-based optimal spare usage algorithm for network restoration study\fP

.SH SYNTAX
\fBrx_mnet  [-ds] <node_file>.txt <net_file>.dat searchStrategy\fP
.sp
.nf
   -d debugging mode.
   -s silence mode dump relax stdin msgs to /dev/null.
   searchStrategy: 1--EXPONENTIAL
                   2--LINEAR_BISECTION
                   3--BISECTION
.fi
.sp
As an example,
to generate the paths with optimal spare usage for all the link breaks in
a network described in file `mci.net', the command is:		
.sp
\fBrx_mnet  -s mcinodes.txt  mci.dat 2 \fP
.sp
Here the \fImcinodes.txt\fP is the node_file that is used to generate 
\fImci.net\fP and \fImci.dat\fP. 
\fImci.net\fP and \fImci.dat\fP are generated by 
\fBnps2net\fP and \fBnps2dat\fP respectively.
The -s means silence mode and ask \fBrx_mnet\fP to print
out as few progress report as possible. `2' at the end of the command line
specifies the use of Linear Bisection searching strategy in case where 
\fBrelax\fP can not found the solution with a bandwidth demand.  
Linear Bisection searching strategy was found to perform better in most cases.
.SH DESCRIPTION
.B rx_mnet  
is a front end interface to the \fBrelax\fP fortran code, which implements
the relaxation based minimum cost flow algorithm. It reads in .dat file,
prepares input data files for \fBrelax\fP on every span cut, and 
launch the \fBrelax\fP execution.
In case where the bandwidth request submitted to \fBrelax\fP is too high,
\fBrelax\fP simply returns "not found" message.  \fBrx_mnet\fP then uses
one of efficient search strategies selected by the user to reduce the 
bandwidth request and found the optimal bandwidth that can be achieved
given the existing spares in the network.  For each span cut, \fBrelax\fP 
generates a file with the restoration paths, the optimal bandwidth found,
and the optimal spare used.  Due to the nature of fortran code, the 
node ID has to be a positive number. To avoid the situation where the 
original node name in the node file may be 0, we choose to use the internal
node ID to specify the span in .dat file.  \fBrx_mnet\fP takes care of
mapping the internal node IDs in the resulting file of \fBrelax\fP into
the corresponding node names using the information in the node_file. 
\fBrx_mnet\fP saves the resulting files in <net_file>.dat.ls directory.
They will be read by the simulators to compute the PNE and PLE numbers
for evaluating the efficiency of the network restoration method.
\fBrx_mnet\fP also generates a summary information file, \fI<net_file>.out\fP,
for the results of all span cuts.
.SH NODE FILE FORMAT
The first line of the node file contains the name of the file.
The rest of line entries in the node file contain four fields, which are
respectively:  `sequenceNo',  `nodeID',  `V', and  `H'.  V and H represent
the vertical and horizontal coordinates of the node.  The sequence
number starts with 1 for the first entry, and is incremented for each
additional entry.  The nodeID is the string name of the node to be specified.
The current upper limit is 300 nodes and 1500 spans.
One node specification entry per line.

.nf
Field Description    			Data Type                  Usual Value  
---------------------------------------------------------------------------
Node sequence number			POSITIVE INTEGER            
Network node ID      			STRING, size 15            1-3 digits
Vertical coordinate  			REAL                       or latitude
Horizontal coordinate			REAL                       or longitude

.fi
.SH RELAX .dat FILE FORMAT
The first line of the relax data file contains a number indicating whether
this network is bidirectional or not. 0 for non-bidirectional, others for 
directional.
The second line of the relax data file contains the number of the nodes in
the network. The third line of the relax data file contains the number of
uni-directional link specification lines.
Each link specification line has six fields: link sequence #, originating node
ID, terminating node ID, link cost, # of spares, # of working channels.

.nf
Field Description   			Data Type        
---------------------------------------------------------------------------
Link Sequence #					POSITIVE INTEGER
Originating Node Internal ID	POSITIVE INTEGER           1-3 digits
Terminating Node Internal ID	POSITIVE INTEGER           1-3 digits
Link Cost						INTEGER
Number of Spares				POSITIVE INTEGER
Number of Working Channels		POSITIVE INTEGER
.fi
.sp
Note that here the internal node ID is used. It corresponds to the
sequence number of the node in the node file or the internal node ID in .net
file. \fBrelax\fI, a fortran code,  will not accept 0 as internal node
ID due to the array indexing.  This is also the reason why the node file is 
needed as one of the input parameter in \fBrx_mnet\fP to convert the results
expressed in terms of internal node IDs back to the equivalent string names.
The order of spare and working channels is also different from that
in .net file.  For a bi-directional link network, the file lists two
uni-directional links of the same span as a pair.
.SH BUG REPORT
Please send bug reports, including the platform on which 
.B rx_mnet 
is executed and the specific scenario which produced the bug,
to chow@quandary.uccs.edu.  We will try to reply as early as possible.
