.TH shnnew n  "15 Jul 95"  "Self-Healing Network Simulator"  "NETRESTORE  Reference Manual"

.SH NAME
\fBshnnew   \-   New Self-Healing Network Simulator\fP

.SH SYNTAX
\fBshnnew  <net_file_name>.net  /b=nodeX\-nodeY  [ /options ]\fP

Simulation Options Section (below) explains available options.  As an example,
to simulate link failure between nodes N03 and N08 with 
repeat limit of 14 and network restoration signaling channels operated at 
192 kbps
in a network described in file `my_network.net', the command is:		

.in 1i
\fBshnnew   my_network.net   /b=N03\-N08   /r=14   /x=192\fP

.SH DESCRIPTION
.B shnnew  
is a distributed network restoration simulator which implements an 
enhanced version of Grover's Self Healing Network algorithm.  
Enhancements
\ 
1. permit the simulation of large networks, 
\ 
2. speed up signature cancellation,
\ 
3. avoid old signature chasing (i.e. chasing bandwidth having 
the same index which just got released),
\ 
4. and support simulation parameters such as node processing time, 
link delay as derived from node coordinates or actual span distances, 
and working and spare channels.  The simulator operates
on a network description file.  The user specifies a network link 
fault as well as desired options, and the simulator provides a step-by-step
profile of the recovery process including the restored bandwidth, 
percentage of the restoration level, and associated time at which that level of 
restoration was achieved.

The NETRESTORE family of tools includes:
\ 
network restoration simulators such as 
\ 
.BR "fitness" ,
.BR "rreact" ,
.BR "twoprong" ,
and 
.BR "shnnew" ;
a relax-based optimal spare algorithm called 
.BR "rx_mnet" ;
\ 
and network file conversion utilities to facilitate the creation of
network description files, such as
\ 
.BR "nps2net" ,
.BR "np2net" ,
.BR "ns2net" ,
.BR "nps2dat" ,
.BR "np2dat" ,
and
.BR "ns2dat" .

.SH SIMULATION SCENARIO
In order to analyze a network using the 
.B shnnew
simulator, a four step sequence will generally be followed.  Each of the
steps below is explained in more detail in later sections by the
same name.
.TP 15
1)  Generate Network Description File
.TP 15
2)  Select Simulation Options
.TP 15
3)  Assemble Simulation Command with Options, Run Simulation
.TP 15
4)  Interpret Simulation Output.

'bp
.SH NETWORK FILE GENERATION
A set of companion tools is available to facilitate
the conversion of existing network files into a NETRESTORE file format shared 
by 
.BR "fitness" 
and other NETRESTORE simulators. 
They provide the basic statistics of the generated network file and 
generate the script file for the simulation of each link restoration in the 
network.  A required input parameter for node processing delay is shared by
all these file conversion utilities.
.BR "nps2net" 
takes the node, path, and spare files as input parameters. 
.BR "np2net"
takes the node and path files as input parameters, and allows users to specify
the spare to working ratio for every span as an additional input parameter.
The node file contains a list of node specification lines with four fields: 
sequence number, node ID, V and H coordinates.
The path file contains the current traffic pattern with a node list of each
of working path followed by `45', DS3 bandwidth, in each line.
These two programs correlate the working  and spare channels in each span, and
the internal ports of each node from the path file.
.BR "ns2net"
takes the node and span files as input parameters. 
The span file contains
the span specification lines, which specify the working and span channels in
each span, and the span distance in kilo meters. 
The 
.BR "ns2net"
correlates the number of input ports in each node from the span 
files.
Check the `man page' for  
.BR "nps2net" ,   
.BR "np2net" ,
and 
.BR "ns2net" .
The
.B nps2net
file is located in   \fI$NETRES/bin\fP.  The command for creating mci.net is: 
.sp
\fBnps2net  mcinodes.txt  mcipaths.txt  mcispares.txt  mci.net  0.01\fP
.sp
Three files are required as input (nodes, paths, and spares) 
followed by the name of the target .net file and
the default message processing delay (in seconds).

To change the message processing time of a specific node,
the node specific repeat limit, or the span distance
of a span, users are referred to the `man page' for the 
.BR "NETRESTORE file format" .
The current version of simulators is case sensitive on the .net filename.
Please use all lower cases for the .net filenames.
.SH SIMULATION OPTIONS
Options and parameters available with \fBshnnew\fP are outlined below.  Each 
option is invoked using the first character (lower case) of the option 
in question.

.B 1TSTIME
\-   This option relates to the transmission time of each individual
signature.  For those
wishing a more fine grained analysis, the node signature transmission
time can be set to something other than the default 0.0 seconds.  For
example, a delay of 2 msec would be set using:     /1=0.002

.B 2INDEX
\-   Refractive index of the optical fiber can be specified with this
option, which defaults to 1.5, corresponding to a 5 microsecond
delay per Km.  It may be changed to 1.4 using:     /2=1.4

.B BREAK
\-   Break is the means by which link failures are specified.  
The format is /b=<node1>\-<node2>.  
This is a \fIrequired\fP parameter.  The user specifies the end nodes using 
the same names as are contained in the companion `.net' network file.        
Note that the name of a node, N07, is different from N7 and node 00 is 
interpreted different from node 0. 
For the break between N02 and N00, use:
	 /b=N02-N01.

.B CROSS_CONNECT_TIME
\-   This is the time delay for a node to effect DCS cross connect.  A default time of  0.01 seconds is assumed.  An 8 msec time would be specified as:     /c=0.008

.B DEBUG
\-   The DEBUG option is not interactive, but simply provides more
detail on the restoration process.  Specifically, it includes the
output of additional simulation information to a log file (default
NETSIM.OUT), a complete
copy of the path listing as found in the .net file, as well as a list of
specific path IDs which were disrupted.  Details of the broadcast phase
are also included.  To invoke this option, add the  `/d' qualifier to the
simulation command.

.B INTERACTIVE
\-   More extensive diagnosis information on fault recovery progress is 
possible using the interactive mode.  Details of events within each node are
reported, for example.  The user is given the option to continue
examining events, abort, or skip `n' events before stopping for another
visual check.  Interactive operation is invoked using:     /i

.B MSG_LENGTH
\-   Restoration time depends on the average length (bytes) 
of restoration messages exchanged by distributed nodes.  The
default message length is 8 bytes.  Restoration message length could be changed to 12 bytes using:     /m=12

.B OUT
\-   Simulation results are stored in a default log file called 
NETSIM.OUT.  An alternative file name can be specified using:	/out=<file_name>

.B PPTOVERRIDE
\-   The default node processing time for each signature
(distinct from 1STSTIME) used in
".net" files created with NETRESTORE translation utilities is ten msec.
This overrides the node processing time of individual nodes, read in as
the third parameter of the first line of each node description
in the .net file.  To override the node processing time 
to 8 msec, use:     /p=0.008

.B REPEATS
\-   This parameter represents the maximum repeat limit or hop count limit
(13 is default) for restoration requests in the broadcast phase.  
It is observed that increasing repeat limit does not guarantee the increase in
restoration level with a real-time constrain. The increase message volume 
may cause network congestion and results in slower restoration time and lower
restoration level.
Specify 9 hop limit as:     /r=9

.B STOPTIME
\-   Upper limit on simulation runtime (sec).  Default value 
is ten (10.0) seconds.  A five second runtime limit is 
specified as:     /s=5.0

.B TIMEOUT
\-   In the event that the Sender and Chooser are unable to achieve full
restoration, the Sender will time-out and cancel the restoration
request, releasing reserved bandwidth.  A default timeout value
of ten (10.0) seconds is assumed.  To get a timeout of 5 seconds, use:     /t=5.0

.B WRITE_LINK
\-   The default is to use the link distance field 
from the .net file.  Specifying  /w  as an option will cause the 
simulator to calculate the link distance 
(and hence link delay) according to V&H or pseudo latitude 
and longitude coordinates as appropriate. 

.B X_SPEED
\-   Transmission speed (kbps) for restoration messages on all links can
be set with this option.  A default value of  8 kbps  is assumed.  A transmission speed of 384 kbps is specified as:     /x=384

.BP
.SH SIMULATION OUTPUT INTERPRETATION
The first portion of the simulation output documents the principal
options selected for this specific run.  Next,
.B shnnew
reports status (4 phases) as it checks the integrity of the .net file,
and ends by indicating the number of active paths before link failure.
Simulation output starts following the statement "SHN algorithm
results...".  

The end nodes of the user specified link failure are
listed, followed by the bandwidth needed for full restoration.

Restoration progress is reported by way of two record types.  An example
of the first record type:

msg_count=00100@ 0.05232 sec Buf#=00102 bwRes=0 
                                         SenderRes=0000 ChooserRes=0002
.TP 0
The first field reports a count of restoration messages which the 
simulator has
processed and the associated elapsed restoration time in
seconds.  Buf# reports the cumulative number of pending restoration 
messages buffered by the nodes.  bwRes reports the cumulative bandwidth restored at this point.  Finally, the amount of reserved bandwidth from the perspective
of the Sender and Chooser is given.

The second record type used to report simulation progress highlights a
path restoration event.  An example follows:
.TP 3
signal 114 rstrd at 252.403 msec (  5.56%)--->
(N00(P0053XP00042)-N02(P00037XP00014)-N01(P00037XP00055)).
.TP 0
In this instance, signal 114 is reported as being restored after an 
elapsed time of 252.403 msec.  The accumulated percentage of restoration
level is also
indicated.  Signal 114 corresponds to a specific working path 
listed in the last half
of the .net file.  Here, a restoration path is found that originates 
at node N00, 
passes through node N02, and terminates at node N01.  In fact,
this is what is reported in the last part of the record.  The arrow
simply indicates that the balance of the record appears on the following
line.  DCS cross connect details for signal 114 are given in the last half 
of the record.  For example, in node N00, ports 53 and 42 are connected.

The last several statements in the simulation output provide a summary
of the simulation.  For example, in the simulation run on us.net with N01-N06 
cut, the following statements appears:
.TP 0
Last Non Time Out Event Time = 0.562081
.nf
Total msg_count=991
access file us.dat.ls/N01_N06cut.l
optimal bandwidth that can be restored=7
optimal spare used =39
N01-N06 6/100=  6.00% @0.541085 msgs=985/991 PNE=6/7 PLE=39/33

.fi
"Last Non Time Out Event Time" reports the total
elapsed restoration time, including cleanup traffic.  The total number
of restoration messages corresponding to this time is given by "Total
msg_count".  
The next three lines indicates that the \fBshnnew\fP found the file with
optimal spare usage and read in the optimal restorable bandwidth and
the minimum spares used to achieve the optimal restoration level.

The last line of output again summarizes the end nodes
connected to the failed link, restored bandwidth, the lost bandwidth, 
(also expressed as a percent).  The time at which this level of restoration 
was achieved is reported.  The two numbers after "msgs=" represent 
respectively the number of messages needed to restore bandwidth, and the 
total number of messages required by the algorithm (including cleanup).
PNE stands for Path Number Efficiency. It is a measure for evaluating the
restoration level achieved by a network restoration method.  It contains
two numbers. The first number represents the number of paths restored by
the network restoration method being evaluated. The second number represents
the maximum number of paths that can be restored. It was typically computed by 
an optimal algorithm called \fBrelax\fP and read in by \fBshnnew\fP.   
PLE stands for Path Length Efficiency. It is a measure for evaluating the 
efficiency of a network restoration method in terms of the spare usage. It
contains two numbers. The first number is the total number of spares in 
the paths found by the optimal spare usage algorithm.  The second number
is the total number of spares in the paths found by the network restoration
method being evaluated.  Note that the PLE ratio is only meaningful when
the PNE=1. The higher the PLE ratio the better. 

Here \fBshnnew\fP
only restore 6 channels while the optimal algorithm found 7 channels.
In the N20-N22 cut of 
us.net, the last line of simulation output shows:

N20-N22 12/100= 12.00% @0.990109 msgs=1403/1403 PNE=12/12 PLE=40/44

Here the optimal algorithm found paths that use only 40 spares, 
while \fBshnnew\fP found paths that use 44 spares.

\fBshnnew\fP also appends this summary information line to the file 
\fInetres.shn.res\fP in the working directory.  This allows a script with
all the span breaks to accumulate the summarized network restoration results.
.SH BUG REPORT
Please send bug reports, including the platform on which 
.B shnnew 
is executed and the specific scenario which produced the bug,
to chow@quandary.uccs.edu.  We will try to reply as early as possible.
